%Oscillation score analysis: Compute oscillation score for spikes
%Example of using the ComputeOscillationScore.dll library
%Author: Raul C. Muresan and Ovidiu F. Jurjut, 25.06.2007
%Description: Computes the oscillation score,the confidence of the score estimate, and the frequency of the peak, for a list of trials of a given stimulation condition.
%             It also returns the spectrum of the smoothed, peakless autocorrelogram
%             For details on the oscillation score, check out the paper: "The Oscillation Score: An Efficient Method for Estimating Oscillation Strength in Neuronal Activity" by Muresan RC, Jurjut OF, Moca VV, Singer W, Nikolic D.
%The FFT in the library is computed using FFTW 3.0.1 (http://www.fftw.org)
%The library may only be used for non-profit scientific purposes
%We do not warrant that the library is bug free. Use at your own risk!
%Special thanks go to Mindy Chang for helping with debugging the code

clear all;

%Some example values for the parameters
fSamplingFrequency = 1000.0; %Suppose we have a sampling frequency of 1 Khz.
iTrialLength = 4000;         %We choose a trial length of 4000 ms.
iTrialNumber = 20;           %Suppose we have 20 trials; IMPORTANT: if you use a single trial, the confidence score cannot be computed and hence it will take a value of 0!
fOscillationFreq = 35;       %Suppose we have an oscillation frequency of 35 Hz
Fmin = 30;                   %Suppose we want to look in the gamma-low band 30-50 Hz
Fmax = 50;                   %Suppose we want to look in the gamma-low band 30-50 Hz

%Produce some random time stamps, with an oscillation frequency of fOscillationFreq for the neuron and store them in a list of trials
time=0:1:iTrialLength; %take a time grid
%Compute each trial
for i=1:iTrialNumber
   SubThreshold=sin(2*pi*fOscillationFreq/fSamplingFrequency*time)+randn(1,iTrialLength+1);         
   Trial=find(SubThreshold > 2.7);    % The time stamps are the times when SubThreshold crosses 2.8 
   TrialList{i} = Trial;              % Add the trials to the list
end

%Compute the oscillation score and confidence of the estimate
%The function expects 5 parameters:
% 	1. SamplingFrequency	- Integer Number; Scalar					- The sampling frequency of your time stamps in Hz
%   2. TrialLength          - Integer Number; Scalar                    - The length of the trial in sampling units
% 	3. TrialList			- Cell Array                                - List of vectors containing the time stamps for the trials of the condition to be analyzed
% 	4. Fmin                 - Double Number; Scalar                     - The lower bound for the frequency range of interest
% 	5. Fmax                 - Double Number; Scalar                     - The higher bound for the frequency range of interest
%Return value:
%   Returns:
%       - a vector R(1,3), containing the oscillation score: R(1,1), the confidence of the oscillation score estimate: R(1,2), and the frequency of the peak in the band of interest R(1,3).
%       - a vector S containing the spectrum of the smoothed, peakless, ACH.
%       - a vector ACH containing the ACH.
%       - a vector spACH containing the smoothed, peakless, ACH.

[R S ACH spACH] = ComputeOscillationScoreAndConfidence(fSamplingFrequency,iTrialLength,TrialList,Fmin,Fmax);

%Get the oscillation score, confidence score and frequency of the peak
OscillationScore = R(1,1)
ConfidenceScore = R(1,2)
FrequencyOfThePeak = R(1,3)

%Plot the spectrum of the smoothed, peakless ACH
N = size(S,2);
f=0:fSamplingFrequency/(2*N):fSamplingFrequency/2*(N-1)/N;
figure(1);
plot(f,S);
xlabel('Frequency [Hz]','FontSize',16);
ylabel('Magnitude','FontSize',16);
title('Spectrum of the smoothed, peakless ACH','FontSize',16);

%Plot the ACH and the smoothed, peakless ACH
iCorrelationWindow = floor(size(ACH,2)/2);
t=-iCorrelationWindow:1:iCorrelationWindow;
figure(2);
plot(t,ACH);
xlabel('Time lag [bins]','FontSize',16);
ylabel('Count','FontSize',16);
title('Autocorrelation histogram (ACH)','FontSize',16);
figure(3);
plot(t,spACH);
xlabel('Time lag [bins]','FontSize',16);
ylabel('Count','FontSize',16);
title('Smoothed, peakless ACH','FontSize',16);










