/*
	++ Source:
			ACHOscillationScore.cpp

	++ Description:
			A class that computes the auto-correlogram of a neuron over more trials,
            the oscillation score of the auto-correlogram, the oscillation frequency
            in a given band and evaluates the confidence of the estimation of the score.

	++ History:
			08-07-2007 - Ovidiu F. Jurjut : Created file and added main declarations

	++ Disclaimer:
			The code is free. You may use it freely, with the sole restriction that you do not claim that you wrote it.
			I do not warrant that the code is 100% bug free. Use at your own risk!
*/


#include "ACHOscillationScore.h"
#include "OscillationScore.h"
#include "CrossCorrelation-DD.h"
#include "Constants.h"
#include "Tools.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "mex.h"


/// Constructor
//------------------------------------------------------------------------------
/**
 * Initializes all internal variables
 *
 * @param SamplingFrequency - Sampling frequency of the signal
 */
CACHOscillationScore::CACHOscillationScore(float SamplingFrequency)
{
    iaaTimestampsPerTrials = NULL;
    iaTimeStampsPerTrialCounts = NULL;
    iTrialCount = -1;
    iTrialLength = -1;

    if (SamplingFrequency > 0) fSamplingFrequency = SamplingFrequency;
    else fSamplingFrequency = 1000;
    fLowBoundFrequency = -1;
    fHighBoundFrequency = -1;

    faaAutoCorrelogramsPerTrials = NULL;
    faAutoCorrelogram = NULL;
    faSmoothedAutoCorrelogramWithoutPeak = NULL;
    faAutoCorrelogramFrequencySpectrum = NULL;

    fOscillationScore = -1;
    fConfidenceScore = 0;
    fCoefficientVariation = 9999999;
    fPeakFrequencyInBand = -1;
    fMeanSmoothedSignalWithOutPeak = -1;
    iNeuronSpikeCounts = 0;
    iValidTrials = 0;

    iCorrelationWindow = -1;
    iCorrelogramSize = -1;
    iComputed = 0;

    srand (time(NULL));
}


/// Destructor
//------------------------------------------------------------------------------
/**
 * Cleans all internal variables
 */
CACHOscillationScore::~CACHOscillationScore()
{
    CleanData();
}


/// Compute the auto-correlogram from a set of spikes, the oscillation score of the auto-correlogram and the confidence of estimation
//------------------------------------------------------------------------------
/**
 * @param TimestampsPerTrials - spike timestamps organized by trials; in sampling frequency units
 * @param TimeStampsPerTrialsCounts - timestamps counts for each trial
 * @param TrialCount - number of trials
 * @param TrialLength - trial lenght; in sampling frequency units
 * @param LowBoundFrequency - minimum frequency of interest for calculating the oscillation score
 * @param HighBoundFrequency - maximum frequency of interest for calculating the oscillation score
 * @return Returns 0 on succes and -1 on error
 */
int CACHOscillationScore::ComputeOscillationScore(int** TimestampsPerTrials, int* TimeStampsPerTrialsCounts, int TrialCount, int TrialLength, float LowBoundFrequency, float HighBandFrequency)
{
	int i;
    if((TrialCount <= 0)||(TrialLength <= 0)||(TimestampsPerTrials==NULL)||(TimeStampsPerTrialsCounts==NULL)) return -1;

    CleanData();
    iComputed = 1;

    COscillationScore*  OS = new COscillationScore(fSamplingFrequency);

    // Copying parameters into internal variables
    iaaTimestampsPerTrials = TimestampsPerTrials;
    iaTimeStampsPerTrialCounts = TimeStampsPerTrialsCounts;
    iTrialCount = TrialCount;
    iTrialLength = TrialLength;
    if (SetOscillationScoreFrequencyBounds(LowBoundFrequency,HighBandFrequency) < 0) return -1;

    // Computing correlation window based on lower bound frequency
    iCorrelationWindow = (int)pow(2,ceil(max(log2(3.0*fSamplingFrequency/fLowBoundFrequency),log2((float)fSamplingFrequency/4.0))));
    iCorrelogramSize = 2 * iCorrelationWindow + 1;

    // Computing auto-correlograms per each trial and storing them
    float* faTempCorrelogram;
    faaAutoCorrelogramsPerTrials = new float*[iTrialCount];
    float* faOscillationScoresPerTrials = new float[iTrialCount];
    InitializeArray(faOscillationScoresPerTrials,iTrialCount,(float)-1);
    int iError;
    CCrossCorrelationDD* CC = new CCrossCorrelationDD(iCorrelationWindow,iTrialLength,iError);

    for(i=0; i<TrialCount; i++)
    {
		faaAutoCorrelogramsPerTrials[i] = new float[iCorrelogramSize];
			for(int j=0; j<iCorrelogramSize; j++) faaAutoCorrelogramsPerTrials[i][j] = 0;
			
		if(iaTimeStampsPerTrialCounts[i] != 0)
		{
			CC->ComputeCrossCorrelation(iaaTimestampsPerTrials[i],iaTimeStampsPerTrialCounts[i],iaaTimestampsPerTrials[i],iaTimeStampsPerTrialCounts[i],0);
			faTempCorrelogram = CC->GetCrossCorrelogram();

			for(int j=0; j<iCorrelogramSize; j++)
				faaAutoCorrelogramsPerTrials[i][j] = faTempCorrelogram[j];

			OS->ComputeOscillationScore(faTempCorrelogram,iCorrelogramSize,fLowBoundFrequency,fHighBoundFrequency);
			faOscillationScoresPerTrials[i] = OS->GetOscillationScore();
		}
    }

    // Computing Coefficient of Variation and Confidence Score
    iValidTrials = 0;
    float fMean = 0;
    for(i=0; i<TrialCount; i++)
        if(faOscillationScoresPerTrials[i] >= 0)
        {
            iValidTrials++;
            fMean += faOscillationScoresPerTrials[i];
        }

    if(TrialCount > 1)
    {
        fMean /= TrialCount;
        float fSum = 0;
        for(int i=0; i<TrialCount; i++)
            if(faOscillationScoresPerTrials[i] >= 0)
                fSum += (faOscillationScoresPerTrials[i] - fMean) * (faOscillationScoresPerTrials[i] - fMean);
            else
                fSum += fMean * fMean;
        float fSTDev = (float)sqrt(fSum/(TrialCount-1));

        if(fMean != 0)
        {
            fCoefficientVariation = fSTDev/fMean;
            fConfidenceScore = (float)(1.0/(1.0 + fCoefficientVariation));
        }
    }

    // Computing auto-correlogram for all trials and storing it
    int iBinCounts = 0;
    faAutoCorrelogram = new float[iCorrelogramSize];
    InitializeArray(faAutoCorrelogram,iCorrelogramSize,(float)0);
    for(i=0; i<TrialCount; i++)
	{
        if(faaAutoCorrelogramsPerTrials[i][iCorrelationWindow] > 0)
        {
            iBinCounts++;
            for(int j=0; j<iCorrelogramSize; j++)
                faAutoCorrelogram[j] += faaAutoCorrelogramsPerTrials[i][j];
        }
	}

    if(iBinCounts>0) iNeuronSpikeCounts = (int) faAutoCorrelogram[iCorrelationWindow];

    for(i=0; i<iCorrelogramSize; i++)
    {
        if(iBinCounts > 0)
            faAutoCorrelogram[i] /= (iBinCounts * (float)TrialLength/fSamplingFrequency);
    }

    delete[] faOscillationScoresPerTrials;

    // Computing oscillation score
    OS->ComputeOscillationScore(faAutoCorrelogram,iCorrelogramSize,fLowBoundFrequency,fHighBoundFrequency);

    fOscillationScore = OS->GetOscillationScore();
    fPeakFrequencyInBand = OS->GetPeakFrequencyInBand();

    faTempCorrelogram = OS->GetSmoothedCorrelogramWithoutPeak();
    if(faTempCorrelogram!=NULL)
    {
        faSmoothedAutoCorrelogramWithoutPeak = new float[iCorrelogramSize];
        for(i=0; i<iCorrelogramSize; i++)
            faSmoothedAutoCorrelogramWithoutPeak[i] = faTempCorrelogram[i];
    }

    faTempCorrelogram = OS->GetFrequencySpectrum();
    if(faTempCorrelogram!=NULL)
    {
        faAutoCorrelogramFrequencySpectrum = new float[iCorrelogramSize/2];
        for(i=0; i<iCorrelogramSize/2; i++)
            faAutoCorrelogramFrequencySpectrum[i] = faTempCorrelogram[i];
    }

    fMeanSmoothedSignalWithOutPeak = OS->GetMeanValueOfSmoothedCorrelogramWithoutPeak();

    return 0;
}


/// Return spike counts of neuron
//------------------------------------------------------------------------------
/**
 * @return Returns the total number of spikes of the neurons in all trials
 */
int CACHOscillationScore::GetSpikeCounts()
{
    return iNeuronSpikeCounts;
}


/// Returns the oscillation score
//------------------------------------------------------------------------------
/**
 * @return Returns the oscillation score in the frequency band of interest; returns -1 if the oscillation scores is not defined or not computed
 */
float CACHOscillationScore::GetOscillationScore()
{
    return fOscillationScore;
}


/// Return value the confidence score
//------------------------------------------------------------------------------
/**
 * @return Returns the confidence with which the oscillation score was estimated (reflects how stationary is the signal accross trials). Return value is between 0 and 1 (0 = no confidence, 1 = 100% confidence).
 */
float CACHOscillationScore::GetConfidenceScore()
{
    return fConfidenceScore;
}

/// Return the auto-correlogram computed on all trilas
//------------------------------------------------------------------------------
/**
 * @warning Check for NULL value before using
 * @return Returns a pointer to the auto-correlogram computed on all trials on succes and NULL on error or if auto-crrelogram is not computed.
 *         The result is in coincidences / second.
 */
float* CACHOscillationScore::GetAutoCorrelogram()
{
    return faAutoCorrelogram;
}


/// Return the peak oscillating frequency in the band of interest
//------------------------------------------------------------------------------
/**
 * @return Returns the peak oscillation frequency from the band of interest in the specrum of the auto-correlogram; result is in Hz. Returns -1 if the frequency is not defined or not computed.
 */
float CACHOscillationScore::GetPeakOscillationFrequencyInBand()
{
    return fPeakFrequencyInBand;
}


///Returns the smoothed peakless auto-correlogram computed on all trials
//------------------------------------------------------------------------------
/**
 * @warning Check for NULL value before using
 * @return Return a pointer to the smoothed auto-correlogram on all trials without the central peak; returns NULL on error or if the auto-correlogram is not computed
 */
float* CACHOscillationScore::GetSmoothedAutoCorrelogramWithoutPeak()
{
    return faSmoothedAutoCorrelogramWithoutPeak;
}

/// Return the mean value of the smoothed auto-correlogram without the peak
//------------------------------------------------------------------------------
/**
 * @return Returns the mean value of smoothed peakless auto-correlogram computed on all trials; returns -1 if the mean is not defined or not computed
 */
float   CACHOscillationScore::GetMeanValueOfSmoothedAutoCorrelogramWithoutPeak()
{
    return fMeanSmoothedSignalWithOutPeak;
}


/// Return the frequency spectrum of the auto-correlogram on all trials
//------------------------------------------------------------------------------
/**
 * @warning Check for NULL value before using
 * @return Returns a pointer to the frequency spectrum of the smoothed peakless auto-correlogram; returns NULL on error or if the auto-correlogram is not computed. Result is in units of correlogram frequency / correlogram size
 */
float* CACHOscillationScore::GetFrequencySpectrum()
{
    return faAutoCorrelogramFrequencySpectrum;
}


/// Return the low bound frequency of the band of interest
//------------------------------------------------------------------------------
/**
 * @return Returns the minimum frequency of interest for calculating the oscillations score. Result is in Hz.
 */
float CACHOscillationScore::GetLowBoundFrequency()
{
    return fLowBoundFrequency;
}


/// Return the high bound frequency of the band of interest
//------------------------------------------------------------------------------
/**
 * @return Returns the maximum frequency of interest for calculating the oscillations score. Result is in Hz.
 */
float CACHOscillationScore::GetHighBoundFrequency()
{
    return fHighBoundFrequency;
}


/// Returns the sampling frequency
//------------------------------------------------------------------------------
/**
 * @return Returns the sampling frequency of the signal
 */
float CACHOscillationScore::GetSamplingFrequency()
{
    return fSamplingFrequency;
}


/// Return the correlation window
//------------------------------------------------------------------------------
/**
 * @return Returns the correlation window (this is computed automatically based on your frequency band of interest). Result is in sampling units.
 */
int CACHOscillationScore::GetCorrelationWindow()
{
    return iCorrelationWindow;
}


/// Return the coefficient of variation
//------------------------------------------------------------------------------
/**
 * @return Returns the coefficient of variation of the oscillation scores computed on the single-trial auto-correlograms; returns 9999999 on error or if not computed.
 */
float CACHOscillationScore::GetCoeffcientOfVariation()
{
    return fCoefficientVariation;
}


/// Returns the number of valid trials
//------------------------------------------------------------------------------
/**
 * @return Returns the number of valid trials (trials that have spikes)
 */
int CACHOscillationScore::GetValidTrials()
{
    return iValidTrials;
}


/// Set the low and high bound frequency for computing the oscillation score
//------------------------------------------------------------------------------
/**
 * The oscillation score is computed on the frequency band between LowBoundFrequency and LowBoundFrequency
 * @param LowBoundFrequency - Minimum frequency of interest in Hz for calculating the oscillation score
 * @param HighBoundFrequency - Maximum frequency of interest in Hz for calculating the oscillation score
 * @warning \b AVOID setting wide bands of interest (see paper why)
 * @return Returns 0 on success and -1 if the parameters are smaller than 0, larger than SamplingFrequency/2 or Low>=High
 */
int CACHOscillationScore::SetOscillationScoreFrequencyBounds(float LowBoundFrequency, float HighBoundFrequency)
{
    if((LowBoundFrequency > 0)&&(HighBoundFrequency <= fSamplingFrequency/2)&&(LowBoundFrequency < HighBoundFrequency))
    {
        fLowBoundFrequency = LowBoundFrequency;
        fHighBoundFrequency = HighBoundFrequency;
        return 0;
    }
    else
    {
        fLowBoundFrequency = 1;
        fHighBoundFrequency = fSamplingFrequency/2;
        return -1;
    }
}


///  Set the sampling frequency of the signal
//------------------------------------------------------------------------------
/**
 * @param SamplingFrequency - new sampling frequency
 * @return Returns 0 on succes and -1 on error
 */
int CACHOscillationScore::SetSamplingFrequency(float SamplingFrequency)
{
    if(SamplingFrequency>0)
    {
        fSamplingFrequency = SamplingFrequency;
        return 0;
    }
    else
    {
        fSamplingFrequency = 1000;
        return -1;
    }
}

/// Flag for signaling whether the object is empty or not
//------------------------------------------------------------------------------
/**
 * @return Returns 1 if the oscillation score is computed, 0 otherwise.
 */
int CACHOscillationScore::IsComputed()
{
    return iComputed;
}


// Cleans all internal variables
//------------------------------------------------------------------------------
int CACHOscillationScore::CleanData()
{
	int i;
    try
    {
        if(iComputed)
        {
            iaTimeStampsPerTrialCounts = NULL;
			iaaTimestampsPerTrials = NULL;

            for(i=0; i<iTrialCount; i++)
                 if(faaAutoCorrelogramsPerTrials[i] != NULL) delete[] faaAutoCorrelogramsPerTrials[i];
            if(faaAutoCorrelogramsPerTrials != NULL) delete[] faaAutoCorrelogramsPerTrials;
            faaAutoCorrelogramsPerTrials = NULL;

            iTrialCount = -1;

            iTrialLength = -1;

            fLowBoundFrequency = -1;

            fHighBoundFrequency = -1;

            if(faAutoCorrelogram != NULL) delete[]  faAutoCorrelogram;
            faAutoCorrelogram = NULL;

            if(faSmoothedAutoCorrelogramWithoutPeak != NULL) delete[] faSmoothedAutoCorrelogramWithoutPeak;
            faSmoothedAutoCorrelogramWithoutPeak = NULL;

            if(faAutoCorrelogramFrequencySpectrum != NULL) delete[] faAutoCorrelogramFrequencySpectrum;
            faAutoCorrelogramFrequencySpectrum = NULL;

            fOscillationScore = -1;

            fMeanSmoothedSignalWithOutPeak = 0;

            iNeuronSpikeCounts = 0;

            iValidTrials = 0;

            fConfidenceScore = 0;

            fCoefficientVariation = 9999999;

            fPeakFrequencyInBand = -1;

            iCorrelationWindow = -1;

            iCorrelogramSize = -1;

            iComputed = 0;
        }
        return 0;
    }
    catch(...)
    {
        return -1;
    }
}
