/*
	++ Source:
			ACHOscillationScore.h

	++ Description:
			A class that computes the auto-correlogram of a neuron over more trials,
            the oscillation score of the auto-correlogram, the oscillation frequency
            in a given band and evaluates the confidence of the estimation of the score.

	++ History:
			08-07-2007 - Ovidiu F. Jurjut : Created file and added main declarations

	++ Disclaimer:
			The code is free. You may use it freely, with the sole restriction that you do not claim that you wrote it.
			I do not warrant that the code is 100% bug free. Use at your own risk!
*/

// Doxygen documentation
/**
 * @brief A class that computes the auto-correlogram of a neuron over more trials,
 *           the oscillation score of the auto-correlogram, the oscillation frequency
 *           in a given band and evaluates the confidence of the estimation of the score.
 *
 * @detailed The main function of the class is ComputeOscillationScore. Upon calling this function, the results
 * will be stored in the internal variables of the class. You can access these results using the class's designated functions.
 * Always check the result for default or error values (e.g. NULL, -1) before using them in further computations.
 */
 
#ifndef ACHOscillationScoreH
#define ACHOscillationScoreH


class CACHOscillationScore
{
    private:

        int**   iaaTimestampsPerTrials;                 // Spike timestamps organised by trials; in sampling frequency units
        int*    iaTimeStampsPerTrialCounts;             // Timestamps counts for each trial
        int     iTrialCount;                            // Number of trials
        int     iTrialLength;                           // Trial length in sampling units

        float   fSamplingFrequency;                     // Sampling frequency
        float   fLowBoundFrequency;                     // Minumum frequency of interest, in Hz
        float   fHighBoundFrequency;                    // Maximum frequency of interest, in Hz

        float** faaAutoCorrelogramsPerTrials;           // Auto-Correlograms computed on each trial
        float*  faAutoCorrelogram;                      // Auto-Correlogram computed on all trials that have spikes
        float*  faSmoothedAutoCorrelogramWithoutPeak;   // Smoothed Auto-Correlogram computed on all trials without the peak
        float*  faAutoCorrelogramFrequencySpectrum;     // Frequency spectrum of the peakless smoothed Auto-Correlogram

        float   fOscillationScore;                      // Oscillation score of Auto-Correlogram computed on all trials
        float   fConfidenceScore;                       // Confidence of estimating the oscillation score accross trials; between 0..1
        float   fCoefficientVariation;                  // Coefficient of variation of the oscillation scores accross trials
        float   fPeakFrequencyInBand;                   // Peak oscillationg frequency in the interest band; in Hz
        float   fMeanSmoothedSignalWithOutPeak;         // Mean value of the peakless smoothed auto-correlogram on all trials
        int     iNeuronSpikeCounts;                     // Spike counts of the neuron
        int     iValidTrials;                           // Number of valid trials (trials that have spikes)

        int     iCorrelationWindow;                     // Auto-correlation window
        int     iCorrelogramSize;                       // Size of Auto-Correlogram signal = 2 * iCorrelogramWindow + 1
        int     iComputed;                              // flag that indicates whether the data is computed and available

        int     CleanData();

    public:

        // Constructor
        CACHOscillationScore(float SamplingFrequency);
        // Destructor
        ~CACHOscillationScore();


        // Compute the auto-correlogram from a set of spikes, the oscillation score of the auto-correlogram and the confidence of estimation
        // --- TimestampsPerTrials - Spike timestamps organised by trials; in sampling frequency units
        // --- TimeStampsPerTrialsCounts - timestamps counts for each trial
        // --- TrialCount - number of trials
        // --- TrialLength - trial lenght; in sampling frequency units
        // --- LowBoundFrequency - Minimum frequency of interest for calculating the oscillation score
        // --- HighBoundFrequency - Maximum frequency of interest for calculating the oscillation score
        int     ComputeOscillationScore(int** TimestampsPerTrials, int* TimeStampsPerTrialsCounts, int TrialCount, int TrialLength, float LowBoundFrequency, float HighBandFrequency);

        // Returns the oscillation score in the frequency band of interest
        float   GetOscillationScore();

        // Returns the confidence with which the oscillation score was estimated (reflects how stationary is the signal accross trials)
        float   GetConfidenceScore();

        // Returns the coefficient of variation of the oscillation scores computed on the single-trial auto-correlograms
        float   GetCoeffcientOfVariation();

        // Returns the auto-correlogram computed on all trials; result is in coincidences / second
        float*  GetAutoCorrelogram();

        // Returns the smoothed peakless auto-correlogram computed on all trials
        float*  GetSmoothedAutoCorrelogramWithoutPeak();

        // Returns the mean value of smoothed peakless auto-correlogram computed on all trials
        float   GetMeanValueOfSmoothedAutoCorrelogramWithoutPeak();

        // Returns the frequency spectrum of the smoothed peakless auto-correlogram; result is units of correlogram frequency / correlogram size
        float*  GetFrequencySpectrum();

        // Returns the peak oscillating frequency in the band of interest; in Hz
        float   GetPeakOscillationFrequencyInBand();

        // Returns the minimum frequency of interest for calculating the oscillations score
        float   GetLowBoundFrequency();

        // Returns the maximum frequency of interest for calculating the oscillations score
        float   GetHighBoundFrequency();

        // Returns the sampling frequency of the signal
        float   GetSamplingFrequency();

        // Returns the correlation window (this is computed automatically based on your frequency band of interest)
        int     GetCorrelationWindow();

        // Returns the number of valid trials (trials that have spikes)
        int     GetValidTrials();

        // Returns the total number of spikes of the neurons in all trials
        int     GetSpikeCounts();

        // Set the frequency band of interest for calculating the oscillation score
        // AVOID setting wide bands of interest (see paper why)
        // --- LowBoundFrequency - minumum frequency of interest
        // --- HighBoundFrequency - maximum frequency of interest
        // +++ Returns 0 on succes and -1 if the parameters are smaller than 0, larger than SamplingFrequency/2 or Low>=High
        int     SetOscillationScoreFrequencyBounds(float LowBoundFrequency, float HighBoundFrequency);


        // Set the sampling frequency of the signal
        // --- SamplingFrequency - new sampling frequency
        int     SetSamplingFrequency(float SamplingFrequency);


        // Flag for signaling whether computations have been done
        int     IsComputed();

};


#endif
