/*
	++ Source:
			OscillationScore.cpp

	++ Description:
			A class that computes the oscillation score and oscillation frequency for a correlogram in a given frequency band of interest

	++ History:
			08-07-2007 - Ovidiu F. Jurjut : Created file and added main declarations

	++ Disclaimer:
			The code is free. You may use it freely, with the sole restriction that you do not claim that you wrote it.
			I do not warrant that the code is 100% bug free. Use at your own risk!
*/

#include "OscillationScore.h"
#include "Tools.h"
#include "Constants.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mex.h"

/// Constructor
//------------------------------------------------------------------------------
/**
 * Initializes all internal variables
 *
 * @param SamplingFrequency - Sampling frequency of the signal
 */
COscillationScore::COscillationScore(float SamplingFrequency)
{
    faCorrelogram = NULL;
    faHighSmoothedCorrelogram = NULL;
    faLowSmoothedCorrelogram = NULL;
    faSmoothedCorrelogramWithoutPeak = NULL;

    faFrequencySpectrum = NULL;
    faFrequencySpectrumWithPeak = NULL;

    fSamplingFrequency = SamplingFrequency;
    fLowBoundFrequency = 1;
    fHighBoundFrequency = SamplingFrequency/2;

    fOscillationScore = -1;
    fPeakFrequencyInBand = -1;
    fMeanSmoothedSignalWithOutPeak = -1;
    iCorrelogramSize = -1;
}


/// Destructor
//------------------------------------------------------------------------------
/**
 * Cleans up the used resources
 */
COscillationScore::~COscillationScore()
{
    CleanData();
}


/// Return the computed Oscillation Score
//------------------------------------------------------------------------------
/**
 * @return Returns the oscillation score in the frequency band of interest; returns -1 if the oscillation score is not defined or not computed
 */
float COscillationScore::GetOscillationScore()
{
    return fOscillationScore;
}


/// Return the peak frequency in the specified band
//------------------------------------------------------------------------------
/**
 * @return Returns the peak oscillating frequency in the band of interest in Hz; returns -1 if the peak frequency is not defined or not computed
 */
float COscillationScore::GetPeakFrequencyInBand()
{
    return fPeakFrequencyInBand;
}


/// Returns the correlogram
//------------------------------------------------------------------------------
/**
 * @warning Check for NULL value before using
 * @return Returns a pointer to the internal array that keeps the correlogram; returns NULL on error or if not computed
 */
float* COscillationScore::GetCorrelogram()
{
    return faCorrelogram;
}


/// Returns the correlogram smoothed with the fast gaussian kernel
//------------------------------------------------------------------------------
/**
 * @warning Check for NULL value before using
 * @return Returns a pointer to the correlogram smoothed with the fast gaussian kernel; returns NULL on error or if not computed
 */
float* COscillationScore::GetHighSmoothedCorrelogram()
{
    return faHighSmoothedCorrelogram;
}


/// Returns the correlogram smoothed with the slow gaussian kernel
//------------------------------------------------------------------------------
/**
 * @warning Check for NULL value before using
 * @return Returns a pointer to the correlogram smoothed with the slow gaussian kernel; returns NULL on error or if not computed
 */
float* COscillationScore::GetLowSmoothedCorrelogram()
{
    return faLowSmoothedCorrelogram;
}


/// Returns the smoothed peakless correlogram
//------------------------------------------------------------------------------
/**
 * @warning Check for NULL value before using
 * @return Returns a pointer to the smoothed peakless correlogram; returns NULL on error or if not computed
 */
float* COscillationScore::GetSmoothedCorrelogramWithoutPeak()
{
    return faSmoothedCorrelogramWithoutPeak;
}


/// Returns the frequency spectrum of the smoothed peakless correlogram
//------------------------------------------------------------------------------
/**
 * @warning Check for NULL value before using
 * @return Returns a pointer to the frequency spectrum of the smoothed peakless correlogram; returns NULL on error or if not computed; result is in units of correlogram frequency / correlogram size
 */
float* COscillationScore::GetFrequencySpectrum()
{
    return faFrequencySpectrum;
}


/// Return the frequency spectrum without cutting the peak; ONLY for comparison purposes
//------------------------------------------------------------------------------
/**
 * @warning \b ONLY for comparison purposes, <b> DO NOT </b> use in computations
 * @warning Check for NULL value before using
 * @return Returns a pointer to the frequency spectrum of the fast smoothed correlogram without cutting the peak; returns NULL on error or if not computed; result is in units of correlogram frequency / correlogram size
 */
float* COscillationScore::GetFrequencySpectrumWithPeak()
{
    return faFrequencySpectrumWithPeak;
}


/// Return the mean value of the smoothed correlogram without the peak
//------------------------------------------------------------------------------
/**
 * @return Returns the mean value of the smoothed correlogram without the peak; returns -1 on error or if not computed
 */
float COscillationScore::GetMeanValueOfSmoothedCorrelogramWithoutPeak()
{
    return fMeanSmoothedSignalWithOutPeak;
}


/// Set the frequency band of interest for calculating the oscillation score
//------------------------------------------------------------------------------
/**
 * @param LowBoundFrequency - minumum frequency of interest
 * @param HighBoundFrequency - maximum frequency of interest
 * @warning \b AVOID setting wide bands of interest (see paper why)
 * @return Returns 0 on succes and -1 if the parameters are smaller than 0, larger than SamplingFrequency/2 or Low>=High
 */
int COscillationScore::SetFrequencyBounds(float LowBoundFrequency, float HighBoundFrequency)
{
    if((LowBoundFrequency > 0)&&(HighBoundFrequency <= fSamplingFrequency/2)&&(LowBoundFrequency < HighBoundFrequency))
    {
        fLowBoundFrequency = LowBoundFrequency;
        fHighBoundFrequency = HighBoundFrequency;
        return 0;
    }
    else
    {
        fLowBoundFrequency = 1;
        fHighBoundFrequency = fSamplingFrequency/2;
        return -1;
    }
}


/// Set sampling frequency
//------------------------------------------------------------------------------
/**
 * @param SamplingFrequency - new sampling frequency
 */ 
int COscillationScore::SetSamplingFrequency(float SamplingFrequency)
{
    fSamplingFrequency = SamplingFrequency;
    return 0;
}


/// Compute the oscillation score of a given correlogram
//------------------------------------------------------------------------------
/**
 * @param Correlogram - the correlogram signal
 * @param CorrelogramSize - the size of the auto-correlogram signal (2 * Correlation Window + 1)
 * @param LowBoundFrequency - Minimum frequency of interest for calculating the oscillation score
 * @param HighBoundFrequency - Maximum frequency of interest for calculating the oscillation score
 * @return Returns the oscillation score on success and -1 on error;  on error all internal variables are initialized to default values
 */
float COscillationScore::ComputeOscillationScore(float* Correlogram, int CorrelogramSize, float LowBoundFrequency, float HighBoundFrequency)
{
	int i;
	float fGaussSTDev;

    CleanData();
    iComputed = 1;

    if((Correlogram==NULL)||(CorrelogramSize<=0)) return -1;

    // Setting the frequency band of interest
    if(SetFrequencyBounds(LowBoundFrequency,HighBoundFrequency)) return -1;

    // Store Correlogram and CorrelogramSize in the internal variables of the class
    iCorrelogramSize = CorrelogramSize;
    int iHalfCorrelogramSize = iCorrelogramSize/2;
    faCorrelogram = new float[iCorrelogramSize];
    for(i=0; i<iCorrelogramSize; i++)
        faCorrelogram[i] = Correlogram[i];

    // Smoothing the correlogram with a gaussian kernel with sigma computed on the base of the HighBoundFreq
    faHighSmoothedCorrelogram = new float[iCorrelogramSize];
    fGaussSTDev = (float) (min(2,134.0/(1.5*fHighBoundFrequency))*(fSamplingFrequency/1000.0));
    SmoothGaussian_Float(faCorrelogram, faHighSmoothedCorrelogram, iCorrelogramSize, fGaussSTDev);

    // Smoothing the correlogram with a gaussian kernel with sigma computed on the base of the LowBoundFreq
    faLowSmoothedCorrelogram = new float[iCorrelogramSize];
    fGaussSTDev = (float) (2.0 * 134.0/(1.5*fLowBoundFrequency)*(fSamplingFrequency/1000.0));
    SmoothGaussian_Float(faCorrelogram, faLowSmoothedCorrelogram, iCorrelogramSize, fGaussSTDev);
    
	// Cutting central peak based on the Low smoothed correlogram
    // Finding minimum(trough in case shift predictor was substracted) and maximum(central peak) in the LowSmoothedCorrelogram
    float fMax, fMin;
	int iIndex;
    FindMaxInArray(faLowSmoothedCorrelogram,iCorrelogramSize,0,iCorrelogramSize-1,fMax,iIndex);
    FindMinInArray(faLowSmoothedCorrelogram,iCorrelogramSize,0,iCorrelogramSize-1,fMin,iIndex);
    fMin = min(fMin,0); // in case the shift predictor is not extracted minimum is 0

    // Compute the scaling factor of the Y axis in the correlogram such as to match the units of the time axis
    float fRatio;
    if (fMax - fMin != 0)
        fRatio = iCorrelogramSize/(fMax - fMin);
    else
        if ((faCorrelogram[iHalfCorrelogramSize] == NAN)||(faCorrelogram[iHalfCorrelogramSize] == 0))
        {
            CleanData();
            return -1;
        }
        else
            fRatio = 1;
    
	// Finding cutting points
    int iCutLeft, iCutRight;
    FindLeftRightCutIndexes(faLowSmoothedCorrelogram,iCorrelogramSize,iHalfCorrelogramSize,10,fRatio,iCutLeft,iCutRight);
    // Cutting the peak and filling in with the mean values of the cut points
    faSmoothedCorrelogramWithoutPeak = new float[iCorrelogramSize];
    for(i=0; i<iCorrelogramSize;i++)
        if((i>=iCutLeft)&&(i<=iCutRight))
            faSmoothedCorrelogramWithoutPeak[i] = (faHighSmoothedCorrelogram[iCutLeft] + faHighSmoothedCorrelogram[iCutRight])/2;
        else
            faSmoothedCorrelogramWithoutPeak[i] = faHighSmoothedCorrelogram[i];
    
	// Compute mean of the smoothed autocorrelogram without peak
    fMeanSmoothedSignalWithOutPeak = ComputeMeanOfArray(faSmoothedCorrelogramWithoutPeak,iCorrelogramSize);
    
	// Applying Fast Fourier Transform on smoothed correlogram without peak and storing result in faFrequencySpectrum
    // the convolution window is of Blackman type
    faFrequencySpectrum = new float[iHalfCorrelogramSize];
    FFT_Blackman(faSmoothedCorrelogramWithoutPeak,faFrequencySpectrum,2*iHalfCorrelogramSize);
    
	// also computing the FFT on the signal with the peak for comparison purpouses
    faFrequencySpectrumWithPeak = new float[iHalfCorrelogramSize];
    FFT_Blackman(faHighSmoothedCorrelogram,faFrequencySpectrumWithPeak,2*iHalfCorrelogramSize);
    
	// Compute integral of frequency spectrum
    float fIntegral = 0;
    for(i=0; i<iHalfCorrelogramSize; i++)
        fIntegral += faFrequencySpectrum[i];
    fIntegral /= (float)iHalfCorrelogramSize;
    
	// Search oscillation peak in frequency spectrum
    int iLowFrequencyIndex = round((float)iCorrelogramSize/fSamplingFrequency * fLowBoundFrequency);
    int iHighFrequencyIndex = round((float)iCorrelogramSize/fSamplingFrequency * fHighBoundFrequency);
    float fFrequencyPeakValue = 0;
    int iFrequencyPeakIndex = iLowFrequencyIndex;
    FindMaxInArray(faFrequencySpectrum,iHalfCorrelogramSize,iLowFrequencyIndex,iHighFrequencyIndex,fFrequencyPeakValue,iFrequencyPeakIndex);
    
	// Storing oscillation score and peak frequency
    if(fIntegral>0)
    {
        fOscillationScore = fFrequencyPeakValue / fIntegral;
        fPeakFrequencyInBand = (iFrequencyPeakIndex * (fSamplingFrequency/2))/(float)iHalfCorrelogramSize;
        return fOscillationScore;
    }
    else
    {
        CleanData();
        return -1;
    }
}




// Clean resources
//------------------------------------------------------------------------------
int COscillationScore::CleanData()
{
    if(iComputed)
    {
        delete[] faCorrelogram;
        faCorrelogram = NULL;

        delete[] faHighSmoothedCorrelogram;
        faHighSmoothedCorrelogram = NULL;

        delete[] faLowSmoothedCorrelogram;
        faLowSmoothedCorrelogram = NULL;

        delete[] faSmoothedCorrelogramWithoutPeak;
        faSmoothedCorrelogramWithoutPeak = NULL;

        delete[] faFrequencySpectrum;
        faFrequencySpectrum = NULL;

        delete[] faFrequencySpectrumWithPeak;
        faFrequencySpectrumWithPeak = NULL;

        fOscillationScore = -1;

        fPeakFrequencyInBand = -1;

        fMeanSmoothedSignalWithOutPeak = -1;

        iCorrelogramSize = -1;

        iComputed = 0;

        return 0;
    }
    else
        return -1;
}


/// Flag for signaling whether the computations have been done
//------------------------------------------------------------------------------
/**
 * @return Returns 1 if the oscillation score is computed, 0 otherwise.
 */
int COscillationScore::IsComputed()
{
    return iComputed;
}



// Finds the points where the peak should be cut; returns 0 on success; -1 on error;
//------------------------------------------------------------------------------
int COscillationScore::FindLeftRightCutIndexes(float* faInputArray, int iArraySize, int iCentralIndex, float fLimitAngle, float fScaleY, int &iLeftIndex, int &iRightIndex)
{
    float fDerivative,fLimitDerivativeLeft,fLimitDerivativeRight;

    if((faInputArray==NULL)||(iCentralIndex<=0)||(iCentralIndex>=iArraySize-1)) return -1;
    try
    {
        //Suppose initially that the peak is cut at the central index
        iLeftIndex = iCentralIndex;
        iRightIndex = iCentralIndex;

        //Compute the limit derivatives from the limit angle
        if(cos(fLimitAngle * PI / 180.0) != 0)
        {
            fLimitDerivativeLeft = (float)tan(fLimitAngle * PI / 180.0);
            fLimitDerivativeRight = (float)-tan(fLimitAngle * PI / 180.0);
        }
        else
        {
            fLimitDerivativeLeft  = 0;
            fLimitDerivativeRight = 0;
        }

        //Get pass the central peak on the left
        fDerivative = (faInputArray[iLeftIndex+1] - faInputArray[iLeftIndex]) * fScaleY;
        while((iLeftIndex > 0)&&(fDerivative <= fLimitDerivativeLeft))
        {
            iLeftIndex--;
            fDerivative = (faInputArray[iLeftIndex+1] - faInputArray[iLeftIndex]) * fScaleY;
        }

        //Identify left cut index
        while((iLeftIndex > 0)&&(fDerivative > fLimitDerivativeLeft))
        {
            iLeftIndex--;
            fDerivative = (faInputArray[iLeftIndex+1] - faInputArray[iLeftIndex]) * fScaleY;
        }


        //Get pass the central peak on the right
        fDerivative = (faInputArray[iRightIndex] - faInputArray[iRightIndex-1]) * fScaleY;
        while((iRightIndex < iArraySize-2)&&(fDerivative >= fLimitDerivativeRight))
        {
            iRightIndex++;
            fDerivative = (faInputArray[iRightIndex] - faInputArray[iRightIndex-1]) * fScaleY;
        }

        //Identify right cut index
        while((iRightIndex < iArraySize-2)&&(fDerivative < fLimitDerivativeRight))
        {
            iRightIndex++;
            fDerivative = (faInputArray[iRightIndex] - faInputArray[iRightIndex-1]) * fScaleY;
        }
    }
    catch (...)
    {
        return -1;
    }

    return 0;
}
