/*
	++ Source:
			OscillationScore.h

	++ Description:
            A class that computes the oscillation score and oscillation frequency for a correlogram in a given frequency band of interest

	++ History:
			08-07-2007 - Ovidiu F. Jurjut : Created file and added main declarations

	++ Disclaimer:
			The code is free. You may use it freely, with the sole restriction that you do not claim you wrote it.
			I do not warrant that the code is 100% bug free. Use at your own risk!
*/

// Doxygen documentation
/**
 * @brief A class that computes the oscillation score and oscillation frequency for a correlogram in a given frequency
 * band of interest
 *
 * @detailed The main function of the class is ComputeOscillationScore. Upon calling this function, the results
 * will be stored in the internal variables of the class. You can access these results using the class's designated functions.
 * Always check the result for default or error values (e.g. NULL, -1) before using them in further computations.
 */

#ifndef OscillationScoreH
#define OscillationScoreH


class COscillationScore
{
    private:
        float*  faCorrelogram;                      ///< Array which stores the correlogram signal in sampling frequency units
        float   fLowBoundFrequency;                 ///< Minimum frequency of interest for calculating the oscillation score; in Hz
        float   fHighBoundFrequency;                ///< Maximum frequency of interest for calculating the oscillation score; in Hz
        int     iCorrelogramSize;                   ///< Size of the auto-correlogram array
        float*  faHighSmoothedCorrelogram;          ///< Correlogram smoothed with a fast gaussian kernel computed on the base of the minimum frequency of interest
        float*  faLowSmoothedCorrelogram;           ///< Correlogram smoothed with a slow gaussian kernel computed on the base of the maximum frequency of interest
        float*  faSmoothedCorrelogramWithoutPeak;   ///< Smoothed auto-correlogram with the central peak cut
        float*  faFrequencySpectrum;                ///< Frequency spectrum of the smoothed auto-correlogram without peak
        float*  faFrequencySpectrumWithPeak;        ///< Frequency spectrum of the smoothed auto-correlogram with the peak; used ONLY for comparison purposes
        float   fOscillationScore;                  ///< Oscillation score in the frequency band of interest
        float   fPeakFrequencyInBand;               ///< Peak frequency in the band of interest based on which the oscillation score is computed
        float   fSamplingFrequency;                 ///< Sampling frequency of the signal
        int     iComputed;                          ///< Flag for signaling whether the object is empty or not
        float   fMeanSmoothedSignalWithOutPeak;     ///< Mean value of the faSmoothedCorrelogramWithoutPeak signal

        int     CleanData();
        int     FindLeftRightCutIndexes(float* faInputArray, int iArraySize, int iCentralIndex, float fLimitAngle, float fScaleY, int &iLeftIndex, int &iRightIndex);

    public:
        // Constructor
        COscillationScore(float  SamplingFrequency);

        // Destructor
        ~COscillationScore();

	    // Compute the oscillation score of a given correlogram
        // --- Correlogram - the correlogram signal
        // --- CorrelogramSize - the size of the auto-correlogram signal (2 * Correlation Window + 1)
        // --- LowBoundFrequency - Minimum frequency of interest for calculating the oscillation score
        // --- HighBoundFrequency - Maximum frequency of interest for calculating the oscillation score
        // +++ Returns the oscillation score on success and -1 on error;  on error all internal variables are initialized to defaults
        float   ComputeOscillationScore(float* Correlogram, int CorrelogramSize,  float LowBoundFrequency, float HighBoundFrequency);

        // Return the oscillation score
        float   GetOscillationScore();

        // Returns the correlogram
        float*  GetCorrelogram();

        // Returns the fast smoothed correlogram
        float*  GetHighSmoothedCorrelogram();

        // Returns the slow smoothed correlogram
        float*  GetLowSmoothedCorrelogram();

        // Returns the smoothed peakless correlogram
        float*  GetSmoothedCorrelogramWithoutPeak();

        // Returns the frequency spectrum of the smoothed peakless correlogram
        float*  GetFrequencySpectrum();

        // Returns the frequency spectrum of the fast smoothed correlogram
        float*  GetFrequencySpectrumWithPeak();

        // Returns the peak oscillating frequency in the band of interest
        float   GetPeakFrequencyInBand();

        // Flag for signaling whether computations have been done
        int     IsComputed();

        // Return the mean of the smoothed peakless correlogram
        float   GetMeanValueOfSmoothedCorrelogramWithoutPeak();

        // Set the sampling frequency of the correlogram
        // --- SamplingFrequency - new sampling frequency
        int     SetSamplingFrequency(float SamplingFrequency);

        // Set the frequency band of interest for calculating the oscillation score
        // AVOID setting wide bands of interest (see paper why)
        // --- LowBoundFrequency - minumum frequency of interest
        // --- HighBoundFrequency - maximum frequency of interest
        // +++ Returns 0 on succes and -1 if the parameters are smaller than 0, larger than SamplingFrequency/2 or Low>=High
        int     SetFrequencyBounds(float LowBoundFrequency, float HighBoundFrequency);
};


#endif
