/*
	++ Source:
			Tools.cpp

	++ Description:
			A collection of usefull functions for handling arrays

	++ History:
			08-07-2007 - Ovidiu F. Jurjut : Created file and added main declarations

	++ Disclaimer:
			The code is free. You may use it freely, with the sole restriction that you do not claim that you wrote it.
			I do not warrant that the code is 100% bug free. Use at your own risk!
*/

#define M_PI 3.1415926535897932384626433832795

#include "Constants.h"
#include "Tools.h"
#include <cmath>
#include <windows.h>
#include <Math.h>
#include "fftw3.h"


int round(float x)
{
	int retval;
	__asm
    {
        fld x
	    fistp retval
    }
	return retval;
}

double log2(double x)
{
	return log(x)/log(2);
}


// Smoothing an array with a gaussian kernel of a given STDev
//---------------------------------------------------------------------------
int SmoothGaussian_Float(float* faSourceArray, float* faDestinationArray, int iArraySize, float fGaussSTDEV)
{
    int result = 0;
	int i,j;
    try
    {
        int iKernelWidth = round(3*fGaussSTDEV);
        float* faGaussianKernel = new float[2*iKernelWidth+1];

        // Make the kernel
        for(i= -iKernelWidth; i<=iKernelWidth; i++)
            faGaussianKernel[i+iKernelWidth] = (float)(1.0/(sqrt(2.0*PI) * fGaussSTDEV) * exp(-(float)i*i/(2.0*fGaussSTDEV*fGaussSTDEV)));

        // Apply the gaussian
        float sum;
        for(i=0; i<iArraySize; i++)
        {
            sum = 0;
            for(j= -iKernelWidth; j<=iKernelWidth; j++)
            {
                if(((i+j)>=0)&&((i+j)<iArraySize))
                    sum += faSourceArray[i+j] * faGaussianKernel[j+iKernelWidth];
                else // mirror signal at borders to reduce border effects
                    if((i+j)<0)
                        sum += faSourceArray[-(i+j)] * faGaussianKernel[j+iKernelWidth];
                    else
                        if((i+j)>=iArraySize)
                            sum += faSourceArray[iArraySize - 1 - ((i+j) - iArraySize + 1)] * faGaussianKernel[j+iKernelWidth];
            }
            faDestinationArray[i] = sum;
        }
        delete[] faGaussianKernel;
    }
    catch(...)
    {
        result = -1;
    }

    return result;
}


//// Computes FFT magnitudes, returns 0 on success else <> 0
/**
 * @param faInput: the input buffer
 * @param faOutput: the output buffer where to write the data (only iSize / 2 entries of the buffer are filled)
 * @param iSize: the size of the input buffer
 * @return returns 0 on success else -1
 */
int ComputeFFTMagnitudes(float* faInput,float *faOutput, int iSize)
{
	int result = 0;
    try
    {
		int i;

		fftw_complex *in, *out;
		fftw_plan p;
 
		in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * iSize);
		out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * iSize);
		
		for(i=0;i<iSize;i++)
		{
			in[i][0] = faInput[i];
			in[i][1] = 0;
		}

		p = fftw_plan_dft_1d(iSize, in, out, FFTW_FORWARD, FFTW_ESTIMATE);
    
		fftw_execute(p); /* repeat as needed */
    
		for(i=0;i<(iSize/2);i++)
		{
			faOutput[i] = (float)sqrt(out[i][0] * out[i][0] + out[i][1] * out[i][1]) / (iSize / 2);
		}

		fftw_destroy_plan(p);
		fftw_free(in); fftw_free(out);
    }
    catch(...)
    {
        result = -1;
    }
    return result;
}

///Computes FFT and extracts the real and imaginary parts, returns 0 on success else <> 0
/**
 * @param faInput: the input buffer
 * @param faOutputReal: the output buffer where to write the data for the real part (only iSize / 2 entries of the buffer are filled)
 * @param faOutputImaginary: the output buffer where to write the data for the imaginary part (only iSize / 2 entries of the buffer are filled)
 * @param iSize: the size of the input buffer
 * @return returns 0 on success else -1
 */
int ComputeFFT_Real_Imaginary(float* faInput,float* faOutputReal,float* faOutputImaginary,int iSize)
{
	int result = 0;
    try
    {
		int i;

		fftw_complex *in, *out;
		fftw_plan p;
 
		in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * iSize);
		out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * iSize);
		
		for(i=0;i<iSize;i++)
		{
			in[i][0] = faInput[i];
			in[i][1] = 0;
		}

		p = fftw_plan_dft_1d(iSize, in, out, FFTW_FORWARD, FFTW_ESTIMATE);
    
		fftw_execute(p); /* repeat as needed */
    
		for(i=0;i<(iSize/2);i++)
		{
			faOutputReal[i] = (float) out[i][0];
			faOutputImaginary[i] = (float) out[i][1];
		}

		fftw_destroy_plan(p);
		fftw_free(in); fftw_free(out);
    }
    catch(...)
    {
        result = -1;
    }
    return result;
}

// Fast Fourier Transform on an array with a Blackman smoothing
// SourceArraySize = must be power of 2 to avoid artefacts
// DestinationArray = 1/2 size of SourceArray
//---------------------------------------------------------------------------
int FFT_Blackman(float* faSourceArray, float* faDestinationArray, int iSourceArraySize)
{
    if(iSourceArraySize < 1) return -1;

	int result = 0;
    try
    {

		int i,n_1;
		float* faTemp;

		for(i=0; i<iSourceArraySize/2; i++)
            faDestinationArray[i] = 0;

		faTemp = new float[iSourceArraySize];
		n_1 = iSourceArraySize - 1;

		//Smooth with the Blackman window
		for(i=0; i<iSourceArraySize; i++) faTemp[i] = (float)(0.42 - 0.5*cos(2*M_PI*i/n_1) + 0.08*cos(4*M_PI*i/n_1)) * faSourceArray[i];

		//Compute the FFT
		ComputeFFTMagnitudes(faTemp,faDestinationArray,iSourceArraySize);

		delete[] faTemp;

    }
    catch(...)
    {
        result = -1;
    }
    return result;
}

