/*
	++ Source:
			Tools.h

	++ Description:
			A collection of usefull functions for handling arrays

	++ History:
			08-07-2007 - Ovidiu F. Jurjut : Created file and added main declarations

	++ Disclaimer:
			The code is free. You may use it freely, with the sole restriction that you do not claim that you wrote it.
			I do not warrant that the code is 100% bug free. Use at your own risk!
*/

#ifndef ToolsH
#define ToolsH

/// max function
#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

/// min function
#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

#include <stdlib.h>
#include <math.h>

/// Round function
/**
 * @return Returns the nearest integer to \a x
 */
int round(float x);

/// Log2 function
/**
 * @return Returns the base 2 logarithm of \a x
 */
double log2(double x);


/// Smoothing an array with gaussian kernel of a given STDev
/**
 * @param faSourceArray - the input buffer
 * @param faDestinationArray - the output buffer where to write the result
 * @param iArraySize - the size of the input buffer
 * @param fGaussSTDEV - standard deviation of the gaussian kernel
 * @return returns 0 on success else -1
 */
int     SmoothGaussian_Float(float* faSourceArray, float* faDestinationArray, int iArraySize, float fGaussSTDEV);


/// Computes FFT magnitudes, returns 0 on success else <> 0
/**
 * @param faInput: the input buffer
 * @param faOutput: the output buffer where to write the data (only iSize / 2 entries of the buffer are filled)
 * @param iSize: the size of the input buffer
 * @return returns 0 on success else -1
 */
int		ComputeFFTMagnitudes(float* faInput,float *faOutput, int iSize);


///Computes FFT and extracts the real and imaginary parts, returns 0 on success else <> 0
/**
 * @param faInput: the input buffer
 * @param faOutputReal: the output buffer where to write the data for the real part (only iSize / 2 entries of the buffer are filled)
 * @param faOutputImaginary: the output buffer where to write the data for the imaginary part (only iSize / 2 entries of the buffer are filled)
 * @param iSize: the size of the input buffer
 * @return returns 0 on success else -1
 */
int		ComputeFFT_Real_Imaginary(float* faInput,float* faOutputReal,float* faOutputImaginary,int iSize);


/// Compute Fast Fourier Transform on an array with a Blackman window
/**
 * @param faSourceArray - the input buffer
 * @param faDestinationArray - the output buffer where to write the result; 1/2 size of SourceArray
 * @param iSourceArraySize - the size of the input buffer; must be power of 2 to avoid artefacts
 * @return returns 0 on success else -1
 */
int     FFT_Blackman(float* faSourceArray, float* faDestinationArray, int iSourceArraySize);


/// Initialize the elements of an array to a specified value
/**
 * @param Array - the input buffer
 * @param iArraySize - the size of the input buffer
 * @param Value - the value to which the array is initialized
 */
template <class T> InitializeArray(T* Array, int iArraySize, T Value)
{
    if(iArraySize <= 0) return -1;
    for(int i=0; i<iArraySize; i++)
        Array[i] = Value;
    return 0;
}


/// Find the maximum value in an array
/**
 * @param taArray - the input buffer
 * @param iArraySize - the size of the input buffer
 * @param iLeftIndex, iRightIndex - boundaries between which the maximum value will be looked for
 * @param tMaxValue - returns the maximum value within the specified boudaries
 * @param iMaxValueIndex - returns the index of the maximum value
 * @return Return 0 on success, -1 on error
*/
template <class T> int FindMaxInArray(T* taArray, int iArraySize, int iLeftIndex, int iRightIndex, T &tMaxValue, int &iMaxValueIndex)
{
    if ((taArray==NULL)||(iLeftIndex<0)||(iLeftIndex>iArraySize-1)||(iRightIndex<0)||(iRightIndex>iArraySize-1)||(iLeftIndex>iRightIndex))
        return -1;

    tMaxValue = taArray[iLeftIndex];
    iMaxValueIndex = iLeftIndex;
    for(int i=iLeftIndex+1; i<=iRightIndex; i++)
        if(taArray[i]>tMaxValue)
        {
            tMaxValue = taArray[i];
            iMaxValueIndex = i;
        }
    return 0;
}


/// Find the minimum value in an array
/**
 * @param taArray - the input buffer
 * @param iArraySize - the size of the input buffer
 * @param iLeftIndex, iRightIndex - boundaries between which the minimum value will be looked for
 * @param tMinValue - returns the minimum value within the specified boudaries
 * @param iMinValueIndex - returns the index of the minimum value
 * @return Return 0 on success, -1 on error
 */
template <class T> int FindMinInArray(T* taArray, int iArraySize, int iLeftIndex, int iRightIndex, T &tMinValue, int &iMinValueIndex)
{
    if ((taArray==NULL)||(iLeftIndex<0)||(iLeftIndex>iArraySize-1)||(iRightIndex<0)||(iRightIndex>iArraySize-1)||(iLeftIndex>iRightIndex))
        return -1;

    tMinValue = taArray[iLeftIndex];
    iMinValueIndex = iLeftIndex;
    for(int i=iLeftIndex+1; i<=iRightIndex; i++)
        if(taArray[i]<tMinValue)
        {
            tMinValue = taArray[i];
            iMinValueIndex = i;
        }
    return 0;
}


/// Calculating the mean of the elements of an array
/**
 * @param taArray - the input buffer
 * @param iArraySize - the size of the input buffer
 * @return Returns the mean value on success, -1 on error
 */
template <class T> float ComputeMeanOfArray(T* taArray, int iArraySize)
{
    if (iArraySize<=0) return -1;

    float sum = 0;
    for(int i=0; i<iArraySize; i++)
        sum += taArray[i];
    return sum/iArraySize;
}


/// Calculating standard deviation on the elements of an array
/**
 * @param taArray - the input buffer
 * @param iArraySize - the size of the input buffer
 * @return Returns the standard deviation on success, -1 on error
 */
template <class T> float ComputeSTDevOfArray(T* taArray, int iArraySize)
{
    if (iArraySize<=0) return -1;
    else if (iArraySize==1) return 0;

    float sum = 0;
    for(int i=0; i<iArraySize; i++)
        sum += taArray[i];
    float mean = sum/(1.0*iArraySize);

    sum = 0;
    for(int i=0; i<iArraySize; i++)
        sum += (taArray[i]-mean)*(taArray[i]-mean);

    return sqrt(sum/(1.0*(iArraySize-1)));
}

#endif
